@@ -105,6 +105,7 @@ static const char helpSign[] =
     "\n"
 #ifndef XMLSEC_NO_XMLDSIG    
     "Signature options:\n"
+    "  --output <filename>   write signed document to file <filename>\n"
     "  --ignore-manifests    do not process <Manifest> elements\n"
     "  --fake-signatures     disable actual signature calc for perf tests\n"
 #else  /* XMLSEC_NO_XMLDSIG */
@@ -126,6 +127,8 @@ static const char helpVerify[] =
     "  --print-signature     store and print the pre-signated\n"
     "                        data (<SignedInfo> element)\n"
     "  --print-all           combination of the all \"--print-*\" options\n"
+    "  --print-xml           print the result information in XML format\n"
+    "  --print-to-file <file> print the result to file <file>\n"
     "  --fake-signatures     disable actual signature calc for perf tests\n"
 #else  /* XMLSEC_NO_XMLDSIG */
     "XML Digital Signatures support was disabled during compilation\n"
@@ -137,6 +140,7 @@ static const char helpEncrypt[] =
     "\n"
 #ifndef XMLSEC_NO_XMLENC
     "Encryption options:\n"
+    "  --output <filename>   write encrypted document to file <filename>\n"
     "  --binary <binary>     binary file to encrypt\n"
     "  --xml <file>          XML file to encrypt\n"
 #else /* XMLSEC_NO_XMLENC */
@@ -147,6 +151,7 @@ static const char helpEncrypt[] =
 static const char helpDecrypt[] =
     "Decrypts data from document in \"XML Encryption\" format.\n"
 #ifndef XMLSEC_NO_XMLENC
+    "  --output <filename>   write decrypted document to file <filename>\n"
 #else /* XMLSEC_NO_XMLENC */
     "\n"
     "XML Encryption support was disabled during compilation\n"
@@ -160,6 +165,9 @@ static const char helpNodeSelection[] =
     "  --node-name [<namespace-uri>:]<name>\n"
     "                        set the operation start point to the first node \n"
     "                        with given <name> and <namespace> URI\n"
+    "  --node-xpath <expr>   set the operation start point to the first node \n"
+    "                        selected by the specified XPath expression\n"
+    "  --dtdfile <file>      Load the specified file as the DTD\n"
     "\n";
     
 static const char helpKeysMngmt[] = 
@@ -195,6 +203,8 @@ static const char helpX509[] =
     "  --trusted <file>      load trusted (root) certificate from PEM file\n"
     "  --untrusted <file>    load un-trusted certificate from PEM file\n"
     "  --pwd <password>      the password to use for reading keys and certs\n"
+    "  --verification-time <time> the local time in \"YYYY-MM-DD HH:MM:SS\"\n"
+    "                       format used certificates verification\n"
 #else /* XMLSEC_NO_X509 */
     "x509 certificates support was disabled during compilation\n"
 #endif /* XMLSEC_NO_X509 */        
@@ -226,20 +236,24 @@ typedef struct _xmlSecDSigStatus {
 } xmlSecDSigStatus, *xmlSecDSigStatusPtr;
 
 /**
- * Init/Shutdown
+ * Init/Shutdown, parsing
  */
 int  initXmlsec(xmlsecCommand command);
 void shutdownXmlsec(void);
 int app_RAND_load_file(const char *file);
 int app_RAND_write_file(const char *file);
+int findIDNodes(xmlDtdPtr dtd, xmlDocPtr doc);
 
 
+xmlNodePtr findStartNode(xmlDocPtr doc, const xmlChar* defNodeName, const xmlChar* defNodeNs);
+
 /**
  * Read command line options
  */
 int  readKeyOrigins(char *keyOrigins);
 int  readPEMCertificate(const char *file, int trusted);
 int  readNumber(const char *str, int *number);
+int  readTime(const char* str, time_t* t);
 int  readKeys(char *file);
 int  readPemKey(int privateKey, char *param, char *name);
 int  readHmacKey(char *filename, char *name);
@@ -292,11 +306,15 @@ int decrypt(xmlDocPtr doc);
 xmlSecKeysMngrPtr keyMgr = NULL; 
 xmlSecKeyPtr sessionKey = NULL;
 
+char *output = NULL; 
 char *nodeId = NULL;
 char *nodeName = NULL;
 char *nodeNs = NULL;
+char* nodeXPath = NULL;
 int repeats = 1;
 int printResult = 0;
+int printXml = 0;
+FILE* printFile = NULL;
 clock_t total_time = 0;
 char *global_pwd = NULL;
 int print_openssl_errors = 0;
@@ -305,6 +323,7 @@ int main(int argc, char **argv) {
     int res = 1;
     xmlsecCommand command = xmlsecCommandNone;
     xmlDocPtr doc = NULL;
+    xmlDtdPtr dtd = NULL;
     int i;
     int pos;
     int ret;
@@ -363,14 +382,14 @@ int main(int argc, char **argv) {
 	 * Node selection options 
 	 */
 	if((strcmp(argv[pos], "--node-id") == 0) && (pos + 1 < argc)) {    
-	    if((nodeName != NULL) || (nodeId != NULL)){
+	    if((nodeName != NULL) || (nodeId != NULL) || (nodeXPath != NULL)){
 		fprintf(stderr, "Error: another node selection option present\n");
 		ret = -1;
 	    } else {
 		nodeId = argv[++pos];
 	    }
 	} else if((strcmp(argv[pos], "--node-name") == 0) && (pos + 1 < argc)) {    
-	    if((nodeName != NULL) || (nodeId != NULL)){
+	    if((nodeName != NULL) || (nodeId != NULL) || (nodeXPath != NULL)){
 		fprintf(stderr, "Error: another node selection option present\n");
 		ret = -1;
 	    } else {
@@ -383,6 +402,24 @@ int main(int argc, char **argv) {
 		    nodeNs = NULL;
 		}
 	    }
+	} else if((strcmp(argv[pos], "--node-xpath") == 0) && (pos + 1 < argc)) {    
+	    if((nodeName != NULL) || (nodeId != NULL) || (nodeXPath != NULL)){
+		fprintf(stderr, "Error: another node selection option present\n");
+		ret = -1;
+	    } else {
+		nodeXPath = argv[++pos];
+	    }
+        } else if((strcmp(argv[pos], "--dtdfile") == 0) && (pos + 1 < argc)) {
+            if(dtd != NULL){
+                fprintf(stderr, "Error: DTD already specified\n");
+                ret = -1;
+            } else {
+                dtd = xmlParseDTD(NULL, (const xmlChar*)argv[++pos]);
+                if(dtd == NULL) {
+                    fprintf(stderr, "Could not parse DTD\n");
+                    ret = -1;
+                }
+            }
 	} else 
 
 	/**
@@ -501,6 +538,27 @@ int main(int argc, char **argv) {
 	    ret = readPEMCertificate(argv[++pos], 1);
 	} else if((strcmp(argv[pos], "--untrusted") == 0) && (pos + 1 < argc)) {	
 	    ret = readPEMCertificate(argv[++pos], 0);
+	} else if((strcmp(argv[pos], "--verification-time") == 0) && (pos + 1 < argc)) {
+	    time_t t = 0;
+	     
+	    if(readTime(argv[++pos], &t) >= 0) {
+#ifndef XMLSEC_NO_XMLDSIG
+		if(dsigCtx != NULL) {
+		    dsigCtx->certsVerificationTime = t;		        
+		}  
+#endif /* XMLSEC_NO_XMLDSIG */
+#ifndef XMLSEC_NO_XMLENC
+		if(encCtx != NULL) { 
+		    encCtx->certsVerificationTime = t;		        
+		} 
+#endif /* XMLSEC_NO_XMLENC */
+		if(keyMgr != NULL) {
+    		    xmlSecSimpleKeysMngrSetCertsFlags(keyMgr, X509_V_FLAG_USE_CHECK_TIME);
+		}
+    		ret = 0;
+	    } else {
+    		ret = -1;
+	    }
 	} else 
 
 	/**
@@ -511,6 +569,9 @@ int main(int argc, char **argv) {
 	} else if((strcmp(argv[pos], "--pwd") == 0) && (pos + 1 < argc)) {
 	    global_pwd = argv[++pos];
 	    ret = 0;
+	} else if((strcmp(argv[pos], "--output") == 0) && (pos + 1 < argc)) {
+	    output = argv[++pos];
+	    ret = 0;
 	} else if((strcmp(argv[pos], "--disable-error-msgs") == 0)) {
 	    xmlSecPrintErrorMessages = 0;
 	    ret = 0;
@@ -529,7 +590,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-rsa") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -538,7 +601,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-dsa") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -547,7 +612,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-des3") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -556,7 +623,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-aes128") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -565,7 +634,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-aes192") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -574,7 +645,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else if((strcmp(argv[pos], "--gen-aes256") == 0) && (pos + 1 < argc)) {
 	    xmlSecKeyPtr key;
@@ -583,7 +656,9 @@ int main(int argc, char **argv) {
 	    if(key != NULL) {
 		ret = xmlSecSimpleKeysMngrAddKey(keyMgr, key);
 	    } else {
-		ret = -1;
+		/* ignore error because we just might have the 
+		 algorithm disabled */
+		ret = 0;
 	    }
 	} else 
 	
@@ -618,6 +693,16 @@ int main(int argc, char **argv) {
 	    dsigCtx->storeManifests = 1; 
 	    dsigCtx->storeSignatures = 1; 	    
 	    printResult = 1;
+	} else if((strcmp(argv[pos], "--print-xml") == 0) && (dsigCtx != NULL))  {
+	    printXml = 1;
+	    printResult = 1;
+	} else if((strcmp(argv[pos], "--print-to-file") == 0) && (dsigCtx != NULL) && 
+		  (pos + 1 < argc) && (printFile == NULL)) {
+	    printFile = fopen(argv[++pos], "w");
+	    if(printFile == NULL) {
+		fprintf(stderr, "Error: failed to open result file \"%s\"\n", argv[pos]);
+		ret = -1;
+	    }
 	} else 
 #endif /* XMLSEC_NO_XMLDSIG */
 
@@ -664,7 +749,7 @@ int main(int argc, char **argv) {
 	 * Check for error
 	 */
 	if(ret < 0) {
-	    printUsage(argv[1]);
+	    printUsage(NULL);
 	    goto done;	    
 	}
 	++pos;
@@ -686,12 +771,14 @@ int main(int argc, char **argv) {
 		doc = xmlSecParseFile(argv[pos]);
 	        if(doc == NULL) {
 		    fprintf(stderr, "Error: failed to read XML file \"%s\"\n", argv[pos]);
-		    printUsage(argv[1]);
+		    printUsage(NULL);
 	    	    goto done;
 		}
-	
-    		switch(command) {
-	    
+                if (dtd) {
+                    findIDNodes(dtd, doc);
+		}
+			
+    		switch(command) {	    
 #ifndef XMLSEC_NO_XMLDSIG	    
     		case xmlsecCommandSign:
 		    ret = generateDSig(doc);
@@ -712,7 +799,7 @@ int main(int argc, char **argv) {
 
 		default:
 		    fprintf(stderr, "Error: unknown command\n");
-	    	    printUsage(argv[1]);
+	    	    printUsage(NULL);
 		    goto done;	    
 		}
 	    }
@@ -746,6 +833,9 @@ done:
     if(doc != NULL) {
 	xmlFreeDoc(doc); 
     }
+    if(dtd != NULL) {
+	xmlFreeDtd(dtd);
+    }
     shutdownXmlsec();
     return(res);
 }
@@ -956,6 +1046,35 @@ int  readNumber(const char *str, int *number) {
     return(0);
 }
 
+int  readTime(const char* str, time_t* t) {
+    struct tm tm;
+    int n;
+    
+    if((str == NULL) || (t == NULL)) {
+	return(-1);
+    }
+    memset(&tm, 0, sizeof(tm));
+    tm.tm_isdst = -1;
+    
+    n = sscanf(str, "%4d-%2d-%2d%*c%2d:%2d:%2d", 
+			    &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
+			    &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
+    if((n != 6) || (tm.tm_year < 1900) 
+		|| (tm.tm_mon  < 1) || (tm.tm_mon  > 12) 
+		|| (tm.tm_mday < 1) || (tm.tm_mday > 31)
+		|| (tm.tm_hour < 0) || (tm.tm_hour > 23)
+		|| (tm.tm_min  < 0) || (tm.tm_min  > 59)
+		|| (tm.tm_sec  < 0) || (tm.tm_sec  > 61)) {
+	return(-1);	    
+    }
+
+    tm.tm_year -= 1900; /* tm relative format year */
+    tm.tm_mon  -= 1; /* tm relative format month */
+
+    (*t) = mktime(&tm);
+    return(0);    
+}
+
 int  readPEMCertificate(const char *file, int trusted) {
 #ifndef XMLSEC_NO_X509	    
     int ret;
@@ -1128,6 +1247,41 @@ int readHmacKey(char *filename, char *name) {
 #endif /* XMLSEC_NO_HMAC */
 }
 
+xmlNodePtr findStartNode(xmlDocPtr doc, const xmlChar* defNodeName, const xmlChar* defNodeNs) {
+    xmlNodePtr cur = NULL;
+    
+    if(doc == NULL) {
+	fprintf(stderr, "Error: document is null\n");
+	return(NULL);
+    }
+    
+    if(nodeId != NULL) {
+	xmlAttrPtr attr;
+	    
+	attr = xmlGetID(doc, BAD_CAST nodeId);
+	cur = (attr != NULL) ? attr->parent : NULL;
+    } else if(nodeName != NULL) {
+	cur = xmlSecFindNode(xmlDocGetRootElement(doc), BAD_CAST nodeName, BAD_CAST nodeNs);
+    } else if(nodeXPath != NULL) {
+	xmlXPathContextPtr ctx = NULL;
+	xmlXPathObjectPtr obj = NULL;
+	
+	ctx = xmlXPathNewContext(doc);
+	obj = xmlXPathEval(BAD_CAST nodeXPath, ctx);
+
+	if ((obj != NULL) && (obj->nodesetval != NULL) && (obj->nodesetval->nodeNr > 0)) {
+	    cur = obj->nodesetval->nodeTab[0];
+	}
+	
+	xmlXPathFreeContext(ctx);
+	xmlXPathFreeObject(obj);
+    } else if(defNodeName != NULL) {
+	cur = xmlSecFindNode(xmlDocGetRootElement(doc), defNodeName, defNodeNs);
+    } else {
+	cur = xmlDocGetRootElement(doc);
+    }
+    return(cur);
+}
 
 /**
  * XML Digital Signature
@@ -1178,8 +1332,7 @@ int generateDSig(xmlDocPtr doc) {
     int res = -1;
     clock_t start_time;
 
-    signNode = xmlSecFindNode(xmlDocGetRootElement(doc), 
-			      BAD_CAST "Signature", xmlSecDSigNs);
+    signNode = findStartNode(doc, BAD_CAST "Signature", xmlSecDSigNs);
     if(signNode == NULL) {
         fprintf(stderr,"Error: failed to find Signature node\n");
 	return(-1);
@@ -1201,12 +1354,26 @@ int generateDSig(xmlDocPtr doc) {
         if(string == NULL) {
 	    fprintf(stderr,"Error: failed to dump document to memory\n");
 	    goto done;
-        }    
-	fwrite(string, len, 1, stdout);
+        }
+	
+	if(output) {
+	    FILE* f = fopen(output, "w");
+	    if(f == NULL) {
+		fprintf(stderr,"Error: failed to open output file \"%s\"\n", output);
+		goto done;
+	    }
+	    fwrite(string, len, 1, f);
+	    fclose(f);
+	} else {	    
+	    fwrite(string, len, 1, stdout);
+	}
     }
     res = 0;
     
 done:    
+    if(printFile != NULL) {
+	fclose(printFile);
+    }
     if(string != NULL) {
 	xmlFree(string);        
     }
@@ -1223,8 +1390,7 @@ int validateDSig(xmlDocPtr doc) {
     clock_t start_time;
     int ret;
     	    
-    signNode = xmlSecFindNode(xmlDocGetRootElement(doc), 
-			      BAD_CAST "Signature", xmlSecDSigNs);
+    signNode = findStartNode(doc, BAD_CAST "Signature", xmlSecDSigNs);
     if(signNode == NULL) {
         fprintf(stderr,"Error: failed to find Signature node\n");
 	return(-1);
@@ -1242,7 +1408,13 @@ int validateDSig(xmlDocPtr doc) {
     } 
 	    
     if(printResult) {
-    	xmlSecDSigResultDebugDump(result, stderr);
+	if(printXml) {	
+    	    xmlSecDSigResultDebugXmlDump(result, 
+		    (printFile != NULL) ? printFile : stderr);
+	} else {
+    	    xmlSecDSigResultDebugDump(result, 
+		    (printFile != NULL) ? printFile : stderr);
+	}	
     }	
 	    
     /** 
@@ -1251,8 +1423,8 @@ int validateDSig(xmlDocPtr doc) {
      */
     memset(&status, 0, sizeof(status));
     getDSigResult(result, &status);
-
-    if(repeats <= 1) { 
+    
+    if(repeats <= 1){ 
         
 	fprintf(stderr, "= Status:\n");
 	fprintf(stderr, "== Signatures ok: %d\n", status.signaturesOk);
@@ -1317,20 +1489,7 @@ int encrypt(xmlDocPtr tmpl) {
 	/**
 	 * What do we want to replace?
 	 */    
-	if(nodeId != NULL) {
-	    xmlAttrPtr attr;
-	    
-	    attr = xmlGetID(doc, BAD_CAST nodeId);
-	    cur = (attr != NULL) ? attr->parent : NULL;
-	} else if(nodeName != NULL) {
-	    cur = xmlSecFindNode(xmlDocGetRootElement(doc), BAD_CAST nodeName, BAD_CAST nodeNs);
-	} else {
-	    cur = xmlDocGetRootElement(doc);
-	}
-	
-	/*
-	 * Did we found node?
-	 */    
+	cur = findStartNode(doc, NULL, NULL);
 	if(cur == NULL) {
     	    fprintf(stderr,"Error: empty document for file \"%s\" or unable to find node\n", data);
 	    goto done;    
@@ -1363,7 +1522,17 @@ int encrypt(xmlDocPtr tmpl) {
     	    fprintf(stderr,"Error: failed to dump document to memory\n");
     	    goto done;
         }
-	fwrite(result, len, 1, stdout);
+	if(output) {
+	    FILE* f = fopen(output, "w");
+	    if(f == NULL) {
+		fprintf(stderr,"Error: failed to open output file \"%s\"\n", output);
+		goto done;
+	    }
+	    fwrite(result, len, 1, f);
+	    fclose(f);
+	} else {	    
+	    fwrite(result, len, 1, stdout);
+	}
     }
     res = 0;
 
@@ -1391,7 +1560,7 @@ int decrypt(xmlDocPtr doc) {
     int ret;
     int res = -1;
 
-    cur = xmlSecFindNode(xmlDocGetRootElement(doc), BAD_CAST "EncryptedData", xmlSecEncNs);
+    cur = findStartNode(doc, BAD_CAST "EncryptedData", xmlSecEncNs);
     if(cur == NULL) {
         fprintf(stderr,"Error: unable to find EncryptedData node\n");
 	goto done;    
@@ -1406,16 +1575,31 @@ int decrypt(xmlDocPtr doc) {
     } 
 
     if(repeats <= 1) {
+	FILE* f = stdout;
+	if(output) {
+	    f = fopen(output, "w");
+	    if(f == NULL) {
+		fprintf(stderr,"Error: failed to open output file \"%s\"\n", output);
+		goto done;
+	    }
+	}
+
 	if((encResult != NULL) && encResult->replaced && (encResult->buffer != NULL)) {
-	    ret = xmlDocDump(stdout, doc);    
+	    ret = xmlDocDump(f, doc);    
         } else if((encResult != NULL) && !encResult->replaced) {
     	    ret = fwrite(xmlBufferContent(encResult->buffer), 
-	       xmlBufferLength(encResult->buffer),
-	       1, stdout);		       
+	    		 xmlBufferLength(encResult->buffer),
+	        	 1, f);		       
 	} else {
+	    if(f != stdout) {
+		fclose(f);
+	    }
     	    fprintf(stderr,"Error: bad results \n");
 	    goto done;    
 	}
+	if(f != stdout) {
+	    fclose(f);
+	}
         if(ret < 0) {
 	    fprintf(stderr,"Error: failed to print out the result \n");
 	    goto done;    
@@ -1459,7 +1643,7 @@ xmlSecKeyPtr genRsa(const char *name) {
     key->name = xmlStrdup(BAD_CAST name);
     return(key);
 #else    
-    fprintf(stderr, "Error: DES support was disabled during compilation\n");
+    fprintf(stderr, "Error: RSA support was disabled during compilation\n");
     return(NULL);
 #endif /* XMLSEC_NO_RSA */    
 }
@@ -1662,6 +1846,14 @@ int app_RAND_write_file(const char *file) {
     return 1;
 }
 
+int findIDNodes(xmlDtdPtr dtd, xmlDocPtr doc) {
+    xmlValidCtxt c;
+
+    memset(&c, 0, sizeof(c));    
+    xmlValidateDtd(&c, doc, dtd);
+    return 0;
+}
+
 
 
 
