@@ -50,12 +50,25 @@ static ERR_STRING_DATA xmlSecStrReasons[]= {
   { XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,		"cert verification failed" },
   { XMLSEC_ERRORS_R_CERT_NOT_FOUND,		"cert not found" },
   { XMLSEC_ERRORS_R_CERT_REVOKED,		"cert revoked" },
+  { XMLSEC_ERRORS_R_CERT_ISSUER_FAILED,		"failed to get cert issuer" },
+  { XMLSEC_ERRORS_R_CERT_NOT_YET_VALID,		"cert is not valid yet" },
+  { XMLSEC_ERRORS_R_CERT_HAS_EXPIRED,		"cert has expired" },
   { XMLSEC_ERRORS_R_DSIG_INVALID_REFERENCE,	"invalid reference" },
   { XMLSEC_ERRORS_R_ASSERTION,			"assertion" },
   { XMLSEC_ERRORS_R_DISABLED,			"disabled" },
   { 0,						NULL}
 };
 
+static ERR_STRING_DATA xmlSecStrLib[]= {
+  { ERR_PACK(XMLSEC_ERRORS_LIB,0,0),		"xmlsec routines"},
+  { 0,     					NULL}
+};
+ 
+static ERR_STRING_DATA xmlSecStrDefReason[]= {
+  { XMLSEC_ERRORS_LIB,				"xmlsec lib"},
+  { 0,						NULL}
+};
+
 static void xmlSecErrorsDefaultCallback		(const char* file, int line, 
 				    		 const char* func,
 						 int reason, const char* msg);
@@ -72,7 +85,9 @@ int  xmlSecPrintErrorMessages = 1;	/* whether the error messages will be printed
 void 
 xmlSecErrorsInit(void) {
     ERR_load_crypto_strings();
-    ERR_load_strings(XMLSEC_ERRORS_LIB, xmlSecStrReasons);
+    ERR_load_strings(XMLSEC_ERRORS_LIB, xmlSecStrLib); /* define xmlsec lib name */
+    ERR_load_strings(XMLSEC_ERRORS_LIB, xmlSecStrDefReason); /* define default reason */
+    ERR_load_strings(XMLSEC_ERRORS_LIB, xmlSecStrReasons); 
 }
 
 /** 
