@@ -22,6 +22,7 @@
 #include <openssl/evp.h>
 #include <openssl/pem.h>
 #include <openssl/x509.h>
+#include <openssl/x509_vfy.h>
 #include <openssl/x509v3.h>
 #include <openssl/pkcs12.h>
 
@@ -33,18 +34,6 @@
 #include <xmlsec/x509.h>
 #include <xmlsec/errors.h>
 
-struct _xmlSecX509Data {
-    X509		*verified;
-    STACK_OF(X509) 	*certs;
-    STACK_OF(X509_CRL)  *crls;
-};
-
-struct _xmlSecX509Store {
-    X509_STORE		*xst;
-    STACK_OF(X509)	*untrusted;
-    STACK_OF(X509_CRL)	*crls;
-};
-
 static int		xmlSecX509DataAddCrl		(xmlSecX509DataPtr x509Data,
 							 X509_CRL *crl);
 static int		xmlSecX509DataAddCert		(xmlSecX509DataPtr x509Data,
@@ -52,6 +41,9 @@ static int		xmlSecX509DataAddCert		(xmlSecX509DataPtr x509Data,
 static void		xmlSecX509DebugDump		(X509 *cert, 
 							 FILE *output);
 
+static void		xmlSecX509DebugXmlDump		(X509 *cert, 
+							 FILE *output);
+
 static int		xmlSecX509StoreVerifyCRL	(xmlSecX509StorePtr store, 
 							 X509_CRL *crl);
 
@@ -183,7 +175,7 @@ xmlSecPKCS12ReadKey(const char *filename, const char *pwd) {
  * or NULL if an error occurs.
  */
 xmlSecX509DataPtr	
-xmlSecX509DataCreate(void) {
+xmlSecX509DataCreate() {
     xmlSecX509DataPtr x509Data;
     
     /*
@@ -247,6 +239,7 @@ xmlSecX509DataDup(xmlSecX509DataPtr x509Data) {
 		    "xmlSecX509DataCreate");
 	return(NULL);
     }
+    newX509->certsVerificationTime = x509Data->certsVerificationTime;
     
     /**
      * Duplicate certs
@@ -445,6 +438,37 @@ xmlSecX509DataDebugDump(xmlSecX509DataPtr x509Data, FILE *output) {
     }
 }
 
+/**
+ * xmlSecX509DataDebugXmlDump:
+ * @x509Data: the pointer to #xmlSecX509Data structure.
+ * @output: the pointer to #FILE structure.
+ *
+ * Prints the information about @x509Data to @output in XML format.
+ */ 
+void
+xmlSecX509DataDebugXmlDump(xmlSecX509DataPtr x509Data, FILE *output) {
+    xmlSecAssert(x509Data != NULL);
+    xmlSecAssert(output != NULL);
+
+    
+    if(x509Data->verified != NULL) {
+	fprintf(output, "<X509Data verified=\"yes\">\n");
+	xmlSecX509DebugXmlDump(x509Data->verified, output);
+	fprintf(output, "</X509Data>\n");
+    }
+    if(x509Data->certs != NULL) {
+	int i;
+
+	fprintf(output, "<X509Data verified=\"no\">\n");	
+	for(i = 0; i < x509Data->certs->num; ++i) {
+	    if(((X509**)(x509Data->certs->data))[i] != x509Data->verified) {
+		xmlSecX509DebugXmlDump(((X509**)(x509Data->certs->data))[i], output);
+	    }
+	}
+	fprintf(output, "</X509Data>\n");
+    }
+}
+
 static void
 xmlSecX509DebugDump(X509 *cert, FILE *output) { 
     char buf[1024];
@@ -469,6 +493,29 @@ xmlSecX509DebugDump(X509 *cert, FILE *output) {
     }
 }
 
+static void
+xmlSecX509DebugXmlDump(X509 *cert, FILE *output) { 
+    char buf[1024];
+    BIGNUM *bn = NULL;
+
+    xmlSecAssert(cert != NULL);
+    xmlSecAssert(output != NULL);
+    
+    fprintf(output, "<X509Cert>\n");
+    fprintf(output, "<SubjectName>%s</SubjectName>\n", 
+	 X509_NAME_oneline(X509_get_subject_name(cert), buf, sizeof(buf))); 
+    fprintf(output, "<IssuerName>%s</IssuerName>\n", 
+	 X509_NAME_oneline(X509_get_issuer_name(cert), buf, sizeof(buf))); 
+    bn = ASN1_INTEGER_to_BN(X509_get_serialNumber(cert),NULL);
+    if(bn != NULL) {
+	fprintf(output, "<IssuerSerial>");
+	BN_print_fp(output, bn);
+	BN_free(bn);
+	fprintf(output, "</IssuerSerial>\n");
+    }
+    fprintf(output, "</X509Cert>\n");
+}
+
 /**
  * xmlSecX509DataReadDerCert:
  * @x509Data: the pointer to #xmlSecX509Data structure.
@@ -593,7 +640,7 @@ xmlSecX509DataWriteDerCert(xmlSecX509DataPtr x509Data, int pos) {
 	goto done;
     }
     
-    res = xmlSecBase64Encode(p, size, 0);
+    res = xmlSecBase64Encode(p, size, 60);
     if(res == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
@@ -896,7 +943,7 @@ xmlSecX509StoreDestroy(xmlSecX509StorePtr store) {
 int
 xmlSecX509StoreVerify(xmlSecX509StorePtr store, xmlSecX509DataPtr x509Data) {
     int ret;
-
+    
     xmlSecAssert2(store != NULL, -1);
     xmlSecAssert2(x509Data != NULL, -1);
     
@@ -929,6 +976,8 @@ xmlSecX509StoreVerify(xmlSecX509StorePtr store, xmlSecX509DataPtr x509Data) {
 	X509 *cert;
 	int i;
 	STACK_OF(X509)* certs;
+	X509 *err_cert = NULL;
+        int err = 0, depth;
 
 	/** 
          * dup certs and add untrusted certs to the stack
@@ -984,7 +1033,18 @@ xmlSecX509StoreVerify(xmlSecX509StorePtr store, xmlSecX509DataPtr x509Data) {
 		X509_STORE_CTX xsc; 
     
 		X509_STORE_CTX_init (&xsc, store->xst, cert, certs);
+		if(store->x509_store_flags & X509_V_FLAG_USE_CHECK_TIME) {
+		    X509_STORE_CTX_set_time(&xsc, 0, 
+			x509Data->certsVerificationTime);
+		}
+		if((store->x509_store_flags & (~X509_V_FLAG_USE_CHECK_TIME)) != 0) {
+		    X509_STORE_CTX_set_flags(&xsc, 
+			store->x509_store_flags & (~X509_V_FLAG_USE_CHECK_TIME));
+		}
 		ret = X509_verify_cert(&xsc); 
+		err_cert = X509_STORE_CTX_get_current_cert(&xsc);
+		err	 = X509_STORE_CTX_get_error(&xsc);
+		depth	 = X509_STORE_CTX_get_error_depth(&xsc);
 		X509_STORE_CTX_cleanup (&xsc);  
 
 		if(ret == 1) {
@@ -993,13 +1053,46 @@ xmlSecX509StoreVerify(xmlSecX509StorePtr store, xmlSecX509DataPtr x509Data) {
 		    return(1);
 		} else if(ret < 0) {
 		    xmlSecError(XMLSEC_ERRORS_HERE,
-				XMLSEC_ERRORS_R_CRYPTO_FAILED,
-				"X509_verify_cert - %d", ret);
+			    XMLSEC_ERRORS_R_CRYPTO_FAILED,
+		    	    "X509_verify_cert - %d (%s)", err,
+			    X509_verify_cert_error_string(err));
 		    sk_X509_free(certs);
 		    return(-1);
 		}
 	    }
 	}
+
+	if((err != 0) && (err_cert != NULL)) {
+	    char buf[256];
+	    switch (err) {
+	    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
+		X509_NAME_oneline(X509_get_issuer_name(err_cert), buf, 256);
+		xmlSecError(XMLSEC_ERRORS_HERE,
+			XMLSEC_ERRORS_R_CERT_ISSUER_FAILED,
+		        "error=%d (%s); issuer=\"%s\"", err,
+		        X509_verify_cert_error_string(err), buf);
+		break;
+	    case X509_V_ERR_CERT_NOT_YET_VALID:
+	    case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
+		xmlSecError(XMLSEC_ERRORS_HERE,
+			XMLSEC_ERRORS_R_CERT_NOT_YET_VALID,
+			"error=%d (%s)", err,
+			X509_verify_cert_error_string(err));
+		break;
+	    case X509_V_ERR_CERT_HAS_EXPIRED:
+	    case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
+		xmlSecError(XMLSEC_ERRORS_HERE,
+			XMLSEC_ERRORS_R_CERT_HAS_EXPIRED,
+			"error=%d (%s)", err,
+			X509_verify_cert_error_string(err));
+		break;
+	    default:			
+		xmlSecError(XMLSEC_ERRORS_HERE,
+			XMLSEC_ERRORS_R_CERT_VERIFY_FAILED,
+			"error=%d (%s)", err,
+			X509_verify_cert_error_string(err));
+	    }		    
+	}
 	sk_X509_free(certs);
     }
     return(0);
