@@ -24,7 +24,7 @@ printRes() {
     if [ $? = 0 ]; then
 	echo "   OK"
     else 
-        echo " Fail"
+        echo " Fail ($?)"
     fi
     if [ -f .memdump ] ; then 
 	cat .memdump >> $logfile 
@@ -42,8 +42,8 @@ execDSigTest() {
 
     if [ -n "$3"  -a -z "$PERF_TEST" ] ; then
 	printf "    Create new signature                                 "
-	echo "$xmlsec_app sign $3 $file.tmpl" >> $logfile
-	$VALGRIND $xmlsec_app sign $EXTRA_PARAMS $3 $file.tmpl > $tmpfile 2>> $logfile
+	echo "$xmlsec_app sign $3 --output $tmpfile $file.tmpl" >> $logfile
+	$VALGRIND $xmlsec_app sign --output $tmpfile $EXTRA_PARAMS $3 $file.tmpl >> $logfile 2>> $logfile
 	printRes
 	
 	if [ -n "$4" ] ; then 
@@ -181,6 +181,13 @@ execDSigTest "aleksey-xmldsig-01/xpointer-hmac" \
     "--hmackey $topfolder/keys/hmackey.bin" \
     "--hmackey $topfolder/keys/hmackey.bin" 
 
+execDSigTest "aleksey-xmldsig-01/enveloping-expired-cert" \
+    "--trusted $topfolder/keys/cacert.pem --allowed x509 --verification-time 2002-10-02+10:00:00" 
+
+execDSigTest "aleksey-xmldsig-01/dtd-hmac-91" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd"
 
 execDSigTest "merlin-exc-c14n-one/exc-signature" \
     ""
@@ -198,6 +205,10 @@ execDSigTest "merlin-xpath-filter2-three/sign-spec" \
 echo "--------- Negative Testing: next test MUST FAIL ----------"
 execDSigTest "merlin-xmldsig-twenty-three/signature-x509-crt-crl" \
     "--trusted $topfolder/merlin-xmldsig-twenty-three/certs/ca.pem"
+
+execDSigTest "aleksey-xmldsig-01/enveloping-expired-cert" \
+    "--trusted $topfolder/keys/cacert.pem --allowed x509" 
+
     
 rm -rf $tmpfile
 echo "--- testDSig finished" >> $logfile
