@@ -1,7 +1,8 @@
 #!/bin/sh
 
+topfolder=$1
+xmlsec_app=$2
 
-topfolder=`pwd`
 timestamp=`date +%Y%m%d_%H%M%S` 
 tmpfile=/tmp/testKeys.$timestamp-$$.tmp
 logfile=/tmp/testKeys.$timestamp-$$.log
@@ -19,44 +20,21 @@ printRes() {
     fi
 }
 
-printNotRes() {
-    if [ $? = 0 ]; then
-        echo " Fail"
-    else 
-	echo "   OK"
-    fi
-    if [ -f .memdump ] ; then 
-	cat .memdump >> $logfile 
-    fi
-}
-
 echo "--- testKeys started ($timestamp) ---"
 echo "--- testKeys started ($timestamp) ---" >> $logfile
 
 printf "    Creating new keys                                    "
-$topfolder/testKeys createkeys $keysfile >> $logfile 2>> $logfile
+$xmlsec_app keys \
+    --gen-hmac "test-hmac-sha1" \
+    --gen-rsa "test-rsa" \
+    --gen-dsa "test-dsa" \
+    --gen-des3 "test-des" \
+    --gen-aes128 "test-aes128" \
+    --gen-aes192 "test-aes192" \
+    --gen-aes256 "test-aes256" \
+    $keysfile >> $logfile 2>> $logfile
 printRes 
 
-printf "    Loading it and dumping private keys                  "
-echo > $tmpfile
-$topfolder/testKeys privatekeys --keys $keysfile $tmpfile 2>> $logfile 
-if [ $? = 0 ] ; then 
-    diff $tmpfile $keysfile >> $logfile
-    printRes
-else 
-    echo " Fail"
-fi
-
-printf "    Loading it and dumping public keys                   "
-echo > $tmpfile
-$topfolder/testKeys publickeys --keys $keysfile $tmpfile 2>> $logfile 
-if [ $? = 0 ] ; then 
-    diff $tmpfile $keysfile >> $logfile
-    printNotRes
-else 
-    echo " Fail"
-fi
-
 rm -rf $tmpfile
 echo "--- testKeys finished ---" >> $logfile
 echo "--- testKeys finished ---"
